/*
   This keyboard driven program lets you experiment with orthogonal
   and perspective projection. The 'o' and 'p' keys switch between
   the two projections. The 'x', 'X', 'y', 'Y', 'z' and 'Z' keys
   move the view point around three dimensional space. Near the
   origin is a picture of a cube and a couple of parallel lines.

   This program demonstrates two perspective projections (you switch
   between them by using the keys '1' and '2'). One in which the
   perspective projection's view volume has the exact same parameters
   as the orthogonal projection's view volume (this means that the near
   rectangle of the perspective projection is the same rectangle as the
   near rectangle of the orthogonal projection). In the other, the far
   rectangle of the perspective projection is the same rectangle as the
   far rectangle of the orthogonal projection.
*/
#include <windows.h>
#include "glut.h"

/* Prototypes for the GLUT callback functions */
void display( void );
void keyboard( unsigned char, int, int );

/* global variables */
double xViewer = 0.0, yViewer = 0.0, zViewer = 5.0; /* coordinates of the camera */
int perspective = 1; /* choose between perspective and parallel projections */
int useNear = 1;     /* choose between two different perspective projections */


int main(int argc, char** argv)
{
   /* Standard GLUT initialization */
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB);
   glutInitWindowSize(500, 500);
   glutInitWindowPosition(100, 100);
   glutCreateWindow(argv[0]);
   glutDisplayFunc(display);
   glutKeyboardFunc(keyboard);

   /* Initialize OpenGL state */
   glClearColor(1.0, 1.0, 1.0, 1.0); /* white background */

   glutMainLoop(); /* Enter event loop */
   return 0;       /* We never get here */
}


void display(void)
{
   /* set up the projection */
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   if ( perspective )
   {
      if (useNear == 1)
      {
      /* Give the perspective view volume the same parameters
         as the orthogonal view volume (so the near rectangle
         of the perspective projection equals the near rectangle
         of the orthogonal projection).                          */
         glFrustum(-3.0, 3.0, -3.0, 3.0, 1.0, 20.0);
      }
      else
      {
      /* Or, make the far rectangle of the perspective projection
         equal to the far rectangle of the orthogonal projection.
         If you put your object near the far plane, then the
         perspective projection will approximate orthogonal projection.
         17.06153122 = 2*convert( arctan(3/20), degrees )         */
         gluPerspective(17.06153122, 1.0, 1.0, 20.0);
      }
   }
   else /* use parallel projection */
   {
      glOrtho(-3.0, 3.0, -3.0, 3.0, 1.0, 20.0);
   }

   /* set up viewing */
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   gluLookAt(xViewer, yViewer, zViewer, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0);

   glClear (GL_COLOR_BUFFER_BIT);

   /* draw three "coordinate" axes. */
   glBegin(GL_LINES);
      glColor3f(0.0, 0.0, 0.0); /* draw in black */
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(3.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 3.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 3.0);
   glEnd();

   /* draw two parallel lines going off to "infinity" */
   glBegin(GL_LINES);
      glColor3f(0.0, 1.0, 0.0); /* draw in green */
      glVertex3f(1.0, -1.0, 0.0);
      glVertex3f(1.0, -1.0, -50.0);
      glVertex3f(2.0, -1.0, 0.0);
      glVertex3f(2.0, -1.0, -50.0);
      glVertex3f(1.0,  0.0, 0.0);
      glVertex3f(1.0, -1.0, 0.0);
      glVertex3f(1.0, -1.0, 0.0);
      glVertex3f(2.0, -1.0, 0.0);
   glEnd();

   /* draw a unit cube */
   glBegin(GL_LINES);
      /* back */
      glColor3f(0.0, 0.0, 1.0); /* draw in red */
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(1.0, 0.0, 0.0);

      glVertex3f(1.0, 0.0, 0.0);
      glVertex3f(1.0, 1.0, 0.0);

      glVertex3f(1.0, 1.0, 0.0);
      glVertex3f(0.0, 1.0, 0.0);

      glVertex3f(0.0, 1.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);

      /* bottom */
      glColor3f(1.0, 0.0, 0.0); /* draw in red */
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 1.0);

      glVertex3f(0.0, 0.0, 1.0);
      glVertex3f(1.0, 0.0, 1.0);

      glVertex3f(1.0, 0.0, 1.0);
      glVertex3f(1.0, 0.0, 0.0);

      glVertex3f(1.0, 0.0, 0.0);
      glVertex3f(0.0, 0.0, 0.0);

      /* top */
      glColor3f(1.0, 0.0, 0.0); /* draw in red */
      glVertex3f(0.0, 1.0, 0.0);
      glVertex3f(0.0, 1.0, 1.0);

      glVertex3f(0.0, 1.0, 1.0);
      glVertex3f(1.0, 1.0, 1.0);

      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(1.0, 1.0, 0.0);

      glVertex3f(1.0, 1.0, 0.0);
      glVertex3f(0.0, 1.0, 0.0);

      /* left */
      glColor3f(1.0, 0.0, 0.0); /* draw in red */
      glVertex3f(0.0, 0.0, 0.0);
      glVertex3f(0.0, 1.0, 0.0);

      glVertex3f(0.0, 1.0, 0.0);
      glVertex3f(0.0, 1.0, 1.0);

      glVertex3f(0.0, 1.0, 1.0);
      glVertex3f(0.0, 0.0, 1.0);

      glVertex3f(0.0, 0.0, 1.0);
      glVertex3f(0.0, 0.0, 0.0);

      /* right */
      glColor3f(1.0, 0.0, 0.0); /* draw in blue */
      glVertex3f(1.0, 0.0, 0.0);
      glVertex3f(1.0, 1.0, 0.0);

      glVertex3f(1.0, 1.0, 0.0);
      glVertex3f(1.0, 1.0, 1.0);

      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(1.0, 0.0, 1.0);

      glVertex3f(1.0, 0.0, 1.0);
      glVertex3f(1.0, 0.0, 0.0);

      /* front */
      glColor3f(0.0, 0.0, 1.0); /* draw in blue */
      glVertex3f(0.0, 0.0, 1.0);
      glVertex3f(1.0, 0.0, 1.0);

      glVertex3f(1.0, 0.0, 1.0);
      glVertex3f(1.0, 1.0, 1.0);

      glVertex3f(1.0, 1.0, 1.0);
      glVertex3f(0.0, 1.0, 1.0);

      glVertex3f(0.0, 1.0, 1.0);
      glVertex3f(0.0, 0.0, 1.0);
   glEnd();

   glFlush(); /* clear buffers */
}//display()


void keyboard(unsigned char key, int x, int y)
{
   switch (key)
   {
      case 'x':
         xViewer+=0.5;
         break;
      case 'X':
         xViewer-=0.5;
         break;
      case 'y':
         yViewer+=0.5;
         break;
      case 'Y':
         yViewer-=0.5;
         break;
      case 'z':
         zViewer+=0.5;
         break;
      case 'Z':
         zViewer-=0.5;
         break;
      case 'o':
         perspective = 0;
         break;
      case 'O':
         perspective = 0;
         break;
      case 'p':
         perspective = 1;
         break;
      case 'P':
         perspective = 1;
         break;
      case '1':
         useNear = 1;
         break;
      case '2':
         useNear = 0;
         break;
      default:
         break;
   }
   printf("xViewer = %f, yViewer = %f, zViewer = %f\n", xViewer, yViewer, zViewer);
   glutPostRedisplay();
}//keyboard()
